#!/bin/sh 
# 
# Compilation, e.g., on T3E 900, mpt.1.4.0.4
# ----------- 

#     mpicc -c -DWITH_MYREDUCE mpi_bench5.c

#     mpicc -O3 -DBYTE_MSG -DUSE_HALF -c myreduce_4algo.c
#     mpicc -o mpi_bench5_halving mpi_bench5.o myreduce_4algo.o
#
#     mpicc -O3 -DBYTE_MSG -DUSE_TREE -c myreduce_4algo.c
#     mpicc -o mpi_bench5_bin_tree mpi_bench5.o myreduce_4algo.o
#
#     mpicc -O3 -DBYTE_MSG -DUSE_BLCK -c myreduce_4algo.c
#     mpicc -o mpi_bench5_bin_blocks mpi_bench5.o myreduce_4algo.o
#
#     mpicc -O3 -DBYTE_MSG -DUSE_PAIR -c myreduce_4algo.c
#     mpicc -o mpi_bench5_pairwise mpi_bench5.o myreduce_4algo.o


#     mpicc -O3 -DBYTE_MSG -c myreduce_4algo.c
#     mpicc -o mpi_bench5_4algo mpi_bench5.o myreduce_4algo.o

#     mpicc -o allreduce_protocol -DALLREDUCE_PROTOCOL myreduce_4algo.c
#     mpicc -o    reduce_protocol    -DREDUCE_PROTOCOL myreduce_4algo.c

# Execution
# --------- 

#     nohup ./bench_script_all_sum > out_t3e_all_sum &
#     nohup ./bench_script_red_sum > out_t3e_red_sum &
#     nohup ./bench_script_all_maxloc > out_t3e_all_maxloc &
#     nohup ./bench_script_red_maxloc > out_t3e_red_maxloc &
 
#     mpirun -np 1 ./allreduce_protocol > prot_list_allreduce
#     mpirun -np 1 ./reduce_protocol    > prot_list_reduce

# Other platforms:
# ----------------
# 
#   - Volvox at HLRS with Myrinet and Intel-compiler:
#  
#       USE_INTEL_COMPILER=1 
#       USE_PROG_MPI=mpich-1.2.4..8a-Myrinet-Intel 
#       In myreduce_4algo.c, line 374 commented out:
#            /* case MPI_REPLACE:  this line must be removed on MPI-1-only implementations */
#  
#       mpicc -O3 -ipo -restrict ... 
#  
#       In bench_script_*:    cd ~/myreduce    (the used local directory) 
#                             for NP in 2 3 4 5 6 7 8 9 10 11 12 13 14 
#       pbsnodes -la                               (status and type of each node) 
#       pbsnodes -la | grep free | wc -l           (number of free nodes)
#       pbsnodes -la | grep job-exclusive | wc -l  (number of used nodes)
#       qstat -Qf                                  (limits of the queues) 
#       qstat -a                                   (status of all submitted jobs)
#       qsub -l nodes=14:ia32:PentiumIV:myrinet,walltime=0:10:00 ./bench_script_all_sum

#   - IBM at SDSC:
#
#       e.g. 
#       mpcc -O3 -DBYTE_MSG -DUSE_HALF -c myreduce_4algo.c
#       mpcc -o mpi_bench5_halving mpi_bench5.o myreduce_4algo.o -lm 
#       poe mpi_bench5_halving < input-file-$1-MYallreduce-sum | sed -e "s/^/pair /"
# 
#       Job characteristics for bench_scriptX IBM_SDSC NP004x8 32:
#               #!/usr/bin/csh
#               #@ environment = COPY_ALL; MP_EUILIB=us;
#               XLSMPOPTS=parthds=1:spins=0:yields=0:
#               schedule=static; AIXTHREAD_SCOPE=S;AIXTHREAD_MNRATIO=8:8;
#               MP_SHARED_MEMORY=yes;
#               MP_PULSE=0;
#               #@ output = LL_out.$(jobid)
#               #@ error = LL_err.$(jobid)
#               #@ initialdir = ~ux454174/reduce
#               #@ job_type = parallel
#               #@ network.MPI = css0,not_shared,US
#               #@ node = 4
#               #@ node_usage = not_shared
#               #@ tasks_per_node = 8
#               #@ notification = error 
#               #@ notification = complete
#               #@ wall_clock_limit = 1:00:00 
#               #@ class = normal
#               #@ queue
#               csh
#               bench_scriptX IBM_SDSC NP004x8 32
 

# Analysis:
# --------- 

#     ./analysis.sh 

# Result:
# ------- 

# The following set of files is produced:

# Algorithms BLCK = binary blocks based recursive halving and doubling 
#            HALF = recursive halving and doubling 
#            PAIR = pairwise and ring
#            TREE = binary tree
#            DFLT = the standard (vendor's) MPI_(All)reduce algorithm 
#            BEST = for each input-case, MPI_MY(all)reduce chooses a 'best'
#                   algorithm out of BLCK, HALF, PAIR, TREE, DFLT,
#                   i.e., the 'chosen best' 
#            
# Variables: $HEAD  e.g. out_t3e
#            $RED   'all' or 'red'     (for MPI_Allreduce and MPI_Reduce)
#            $OP    'sum' or 'maxloc'  (for MPI_SUM and MPI_MAXLOC)
# 
# Trailers:  .eps      may be used in papers
#            _sl.eps   thicker lines, may be better in the paper
#            .png      for slides
# 
# Diagrams:  ... for final analysis or publication
#            $HEAD'_'$RED'_'$OP'_'best.eps                   _sl.eps   _sl.png
#             .. which algorithm is the "real best" 
#            $HEAD'_'$RED'_'$OP'_'ratio_dflt.eps             _sl.eps   _sl.png
#             .. how much is the "real best" better then "vendor's algo."  
#            $HEAD'_'$RED'_'$OP'_'ratio_tree.eps             _sl.eps   _sl.png
#             .. how much is the "real best" better then "binary tree"
#            $HEAD'_'$RED'_'$OP'_'ratio_best.eps             _sl.eps   _sl.png
#             .. how much is the "real best" better then "chosen best"
#            $HEAD'_'$RED'_'$OP'_'ratio5_best.eps            _sl.eps   _sl.png
#             .. how much is the "real best incl. vendor" better then "chosen best"
#            $HEAD'_'$RED'_'$OP'_'ratio6_best.eps            _sl.eps   _sl.png
#             .. how much is the "real best incl. vendor and chosen best" 
#                better then "chosen best", i.e. values greater than 1.1 show
#                that "chosen best" does not deliver maximal bandwidth 
#
#            ... verifying, that "chosen best" is really (nearly) the best 
#            $HEAD'_'$RED'_'$OP'_'overCPUs_1024bytes.eps     _sl.eps   _sl.png
#            $HEAD'_'$RED'_'$OP'_'overCPUs_16bytes.eps       _sl.eps   _sl.png
#            $HEAD'_'$RED'_'$OP'_'overCPUs_2048bytes.eps     _sl.eps   _sl.png
#            $HEAD'_'$RED'_'$OP'_'overCPUs_32768bytes.eps    _sl.eps   _sl.png
#            $HEAD'_'$RED'_'$OP'_'overCPUs_4194304bytes.eps  _sl.eps   _sl.png
#            $HEAD'_'$RED'_'$OP'_'overCPUs_524288bytes.eps   _sl.eps   _sl.png
# 
#            ... diagram-series for analysing and choosing best algorithm
#            $HEAD'_'$RED'_'$OP'_'overBytes.eps              _sl.eps 
#            $HEAD'_'$RED'_'$OP'_'overCPUs.eps               _sl.eps 
#
# Datafiles:
#            $HEAD'_'$RED'_'$OP'                     input for analysis
#            $HEAD'_'$RED'_'$OP'_'ByPeTy             sorted by Bytes, #PEs, Algo.
#            $HEAD'_'$RED'_'$OP'_'ByPeTy_best5       best of BLCK, HALF, PAIR, TREE, DFLT
#            $HEAD'_'$RED'_'$OP'_'ByPeTy_best4       best of BLCK, HALF, PAIR, TREE
#            $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_dflt  ratio := best4 / DFLT
#            $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_tree  ratio := best4 / TREE
#            $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_best  ratio := best4 / BEST
#            $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio5_best ratio := best5 / BEST
#            $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio6_best ratio := best6 / BEST
#
# Sources:
#            README                        - readme file
#            mpi_bench5.c                  - benchmark program
#            myreduce_4algo.c              - all new (all)reduce algorithms
#            bench_script_all_maxloc       - scripts to start mpi_bench5
#            bench_script_all_sum          - ...
#            bench_script_red_maxloc       - ...
#            bench_script_red_sum          - ...
#            input-file-MYallreduce-maxloc - input files for the bench_scripts
#            input-file-MYallreduce-sum    - ...
#            input-file-MYreduce-maxloc    - ...
#            input-file-MYreduce-sum       - ...
#            input-file-allreduce-maxloc   - ...
#            input-file-allreduce-sum      - ...
#            input-file-reduce-maxloc      - ...
#            input-file-reduce-sum         - ...
#            analysis.sh                   - the analysis shell script
#            analysis_1pic.sh              - helper files for analysis.sh
#            comp5_all_best.gnuplot        - ...
#            comp5_red_best.gnuplot        - ...
#            comp5_overBytes_head.gnuplot  - ...
#            comp5_all_overBytes_entry.gnuplot - ...
#            comp5_red_overBytes_entry.gnuplot - ...
#            comp5_all_overCPUs_1pic.gnuplot   - ...
#            comp5_red_overCPUs_1pic.gnuplot   - ...
#            comp5_overCPUs_head.gnuplot   - ...
#            comp5_all_overCPUs_entry.gnuplot  - ...
#            comp5_red_overCPUs_entry.gnuplot  - ...
#            comp5_ratio.gnuplot           - ...
#            first_min.awk                 - ...  (default) 
#            first_avg.awk                 - ...  (alternativ: calculates bandwidth 
#                                                  with average execution time
#                                                  instead of minimal execution time)
#            ratio.awk                     - ...
#            sed_cut                       - ...
#            prot_list_allreduce           - listings showing the default-
#            prot_list_reduce              -   decisions for "chosen best"

# tarfile:
#
# tar -cvf myreduce_4algo.tar README mpi_bench5.c myreduce_4algo.c bench_script_all_maxloc bench_script_all_sum bench_script_red_maxloc bench_script_red_sum input-file-MYallreduce-maxloc input-file-MYallreduce-sum input-file-MYreduce-maxloc input-file-MYreduce-sum input-file-allreduce-maxloc input-file-allreduce-sum input-file-reduce-maxloc input-file-reduce-sum analysis.sh analysis_1pic.sh comp5_all_best.gnuplot comp5_red_best.gnuplot comp5_overBytes.gnuplot comp5_overBytes_head.gnuplot comp5_all_overBytes_entry.gnuplot comp5_red_overBytes_entry.gnuplot comp5_all_overCPUs_1pic.gnuplot comp5_red_overCPUs_1pic.gnuplot comp5_overCPUs_head.gnuplot comp5_all_overCPUs_entry.gnuplot comp5_red_overCPUs_entry.gnuplot comp5_ratio.gnuplot first_avg.awk first_min.awk ratio.awk getNP.awk sed_cut prot_list_allreduce prot_list_reduce 
# gzip myreduce_4algo.tar 
 
# Installing the analysis routines in subdirectories:
# 
#   ln ../ana5/*.awk ../ana5/sed_cut ../ana5/*.gnuplot ../ana5/*.sh .
#    or
#   ln ../ana5/analysis5.sh ../ana5/analysis_1pic.sh ../ana5/comp5_all_best.gnuplot ../ana5/comp5_red_best.gnuplot ../ana5/comp5_all_overBytes_entry.gnuplot ../ana5/comp5_red_overBytes_entry.gnuplot ../ana5/comp5_overBytes_head.gnuplot ../ana5/comp5_all_overCPUs_1pic.gnuplot ../ana5/comp5_red_overCPUs_1pic.gnuplot ../ana5/comp5_overCPUs_head.gnuplot ../ana5/comp5_all_overCPUs_entry.gnuplot ../ana5/comp5_red_overCPUs_entry.gnuplot ../ana5/comp5_ratio.gnuplot ../ana5/first_avg.awk ../ana5/first_min.awk ../ana5/getNP.awk ../ana5/ratio.awk ../ana5/reduce_data_analysis.awk ../ana5/sed_cut .
# 
#   ./analysis.sh 'out_my_platform'
#  
#   required input: out_my_platform_all_sum  out_my_platform_all_maxloc
#                   out_my_platform_red_sum  out_my_platform_red_maxloc

# EOR 
 
# Generating the README 
# 
#  \rm -f README; sed -e 's/.//' -e 's/.//' -e '/EOR/,$d' analysis.sh >README
 
 
 

#-------------------
#   Analysis loops
#-------------------

if [ "$1"x == "x" ] ; then export HEAD='out' ; else export HEAD="$1" ; fi
echo $HEAD': analysis of the whole platform'
for OP in 'sum' 'maxloc'
do
 export OP 
 echo ' ' 
 echo ' '$HEAD'_..._'$OP': analysis for reduction operation "'$OP'"'
 for RED in 'all' 'red'
 do 
  export RED 
  if [ "$RED" == "red" ] ; then export RED_NAME="Reduce" ; else export RED_NAME="Allreduce" ; fi
  echo ' ' 
  echo '  '$HEAD'_'$RED'_'$OP': analysis for MPI_'$RED_NAME'(MPI_'$OP$', double)'

#-------------------
#   check correct results of all algorithms
#-------------------

echo '   Checking - looking for error messages in the benchmark output'
echo '     ...' 
grep ' != ' $HEAD'_'$RED'_'$OP
echo '     If there was no error message above this line, then all is okay,'
echo '     otherwise there is a bug in one of the algorithms!!!' 
echo '   ' 

if [ `grep ' != ' $HEAD'_'$RED'_'$OP | wc -l` != "0" ] ; 
then 
  echo 'Errors detected in '$HEAD'_'$RED'_'$OP', therefore'
  echo 'ABORT'
  exit -1 
fi 
 

#-------------------
#   sorting and compressing
#-------------------

echo '   Sorting' 

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy; echo '      ... bytes  ... PEs method "bw MB/s" "min [ms]" "max [ms]" "avg [ms]"' > $HEAD'_'$RED'_'$OP'_'ByPeTy ;  cat $HEAD'_'$RED'_'$OP | grep ' M ' | sed -f sed_cut | sort -u +3n -4 +1n -2 +2 -3 | gawk -f first_min.awk >> $HEAD'_'$RED'_'$OP'_'ByPeTy

#-------------------
#   process range
#-------------------

# may be calculated with 12 or 16 byte lines 
export PEmin=`grep ' 1. By .... PEs best' $HEAD'_'$RED'_'$OP'_'ByPeTy | head -1 | sed -e 's/.* By *//' -e 's/ PEs.*//'`
export PEmax=`grep ' 1. By .... PEs best' $HEAD'_'$RED'_'$OP'_'ByPeTy | tail -1 | sed -e 's/.* By *//' -e 's/ PEs.*//'`
 
echo '   Process range: '$PEmin' .. '$PEmax

export PElow=`  awk "BEGIN{print $PEmin/1.4}"` 
export PEhigh=` awk "BEGIN{print $PEmax*1.4}"` 
export PEhighT=`awk "BEGIN{print $PEmax*1.85}"` 
export PEmax10=`awk "BEGIN{print $PEmax*10 }"` 

#-------------------
#   byte range
#-------------------

export BYmin=`grep ' By *'$PEmin' PEs best' $HEAD'_'$RED'_'$OP'_'ByPeTy | head -1 | sed -e 's/ *//' -e 's/ By.*//'`
export BYmax=`grep ' By *'$PEmin' PEs best' $HEAD'_'$RED'_'$OP'_'ByPeTy | tail -1 | sed -e 's/ *//' -e 's/ By.*//'`
 
echo '   Byte range: '$BYmin' .. '$BYmax

# ===============================================================================================
if [ "$RED" == "red" ] 
then
 
#-------------------
#   best algorithm out of 6:  (see %6 in the following command)
#-------------------

echo '   Best algorithm out of 6'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_best6; sed -e '1s/method/a_method/' $HEAD'_'$RED'_'$OP'_'ByPeTy |  sort +0n -1 +2n -3 +5nr -6 | gawk '{ if ((FNR-1)%6==1 || FNR==1) print $0 }' | sort +4 -5 +0n -1 +2n -3 | sed -e '1s/a_method/best_of_6/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_best6

#-------------------
#   best algorithm out of 5:  (see %5 in the following command)
#-------------------

echo '   Best algorithm out of 5'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_best5; sed -e '1s/method/a_method/' $HEAD'_'$RED'_'$OP'_'ByPeTy | grep -v 'PEs best' |  sort +0n -1 +2n -3 +5nr -6 | gawk '{ if ((FNR-1)%5==1 || FNR==1) print $0 }' | sort +4 -5 +0n -1 +2n -3 | sed -e '1s/a_method/best_of_5/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_best5

    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_best5 ; echo 'L2to32=11.25; L23to256=10.25; Lpair=13.5; set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_best.eps" ; set title "Fastest Protocol for\n'$RED_NAME'('$OP',dbl)" ; set yrange [1:'$PEmax10'] ; load "comp5_red_best.gnuplot" ; q' | gnuplot

    \rm -f $HEAD'_'$RED'_'$OP'_'best_sl.eps; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'best.eps > $HEAD'_'$RED'_'$OP'_'best_sl.eps
    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'best_sl.eps $HEAD'_'$RED'_'$OP'_'best_sl.png

#-------------------
#   Ratio best bandwidth of my 4 algorithms / vendor bandwidth
#-------------------

echo '   Best of new 4 algorithms'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_best4; sed -e '/best/d' -e '/dflt/d' -e '1s/method/a_method/' $HEAD'_'$RED'_'$OP'_'ByPeTy |  sort +0n -1 +2n -3 +5nr -6 | gawk '{ if ((FNR-1)%4==1 || FNR==1) print $0 }' | sort +4 -5 +0n -1 +2n -3 | sed -e '1s/a_method/best_of_4/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_best4

echo '   Ratio := real best / default'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_dflt; grep 'dflt' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/PEs ..../B__ divisor/' | cat - $HEAD'_'$RED'_'$OP'_'ByPeTy_best4 |  sed -e '1s/... by/000 by/' -e 's/PEs/C__/' | sort +0n -1 +2n -3 +3 -4 | gawk -f ratio.awk |  sed -e '1s/000/.../' -e 's/B__ divisor/PEs d:df/' -e 's/.__/PEs/' -e '1s/best_of_4/methd ratio cat/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_dflt
    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_dflt; echo 'set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_ratio_dflt.eps" ; set title "'$RED_NAME'('$OP',dbl)  -  ratio := best bandwidth of 4 new algo.s / vendor'"'"'s bandwidth" ; set yrange ['$PElow':'$PEhighT'] ; load "comp5_ratio.gnuplot" ; q' | gnuplot
    \rm -f $HEAD'_'$RED'_'$OP'_'ratio_dflt_sl.eps ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'ratio_dflt.eps > $HEAD'_'$RED'_'$OP'_'ratio_dflt_sl.eps
    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'ratio_dflt_sl.eps $HEAD'_'$RED'_'$OP'_'ratio_dflt_sl.png


echo '   Ratio := chosen best / default'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_cb_dflt; egrep 'dflt|best|... by' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/PEs dflt/B__ divisor/' -e '1s/... by/000 by/' -e 's/PEs/C__/' | sort +0n -1 +2n -3 +3 -4 | gawk -f ratio.awk |  sed -e '1s/000/.../' -e 's/B__ divisor/PEs d:df/' -e 's/.__/PEs/' -e '1s/method/methd ratio cat/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_cb_dflt
    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_cb_dflt; echo 'set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_ratio_cb_dflt.eps" ; set title "'$RED_NAME'('$OP',dbl)  -  ratio := bandwidth of chosen best / vendor'"'"'s bandwidth" ; set yrange ['$PElow':'$PEhighT'] ; load "comp5_ratio.gnuplot" ; q' | gnuplot
    \rm -f $HEAD'_'$RED'_'$OP'_'ratio_cb_dflt_sl.eps ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'ratio_cb_dflt.eps > $HEAD'_'$RED'_'$OP'_'ratio_cb_dflt_sl.eps
    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'ratio_cb_dflt_sl.eps $HEAD'_'$RED'_'$OP'_'ratio_cb_dflt_sl.png


echo '   Ratio := real best / tree'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_tree; grep 'tree' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/PEs tree/B__ divisor/' | cat - $HEAD'_'$RED'_'$OP'_'ByPeTy_best4 |  sed -e '1s/... by/000 by/' -e 's/PEs/C__/' | sort +0n -1 +2n -3 +3 -4 |  gawk -f ratio.awk | sed -e '1s/000/.../' -e 's/B__ divisor/PEs d:tr/' -e 's/.__/PEs/' -e '1s/best_of_4/methd ratio cat/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_tree
    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_tree; echo 'set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_ratio_tree.eps" ; set title "'$RED_NAME'('$OP',dbl)  -  ratio := best bandwidth of 4 new algo.s / bin-tree'"'"'s bandwidth" ; set yrange ['$PElow':'$PEhighT'] ;  load "comp5_ratio.gnuplot" ; q' | gnuplot
    \rm -f $HEAD'_'$RED'_'$OP'_'ratio_tree_sl.eps ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'ratio_tree.eps > $HEAD'_'$RED'_'$OP'_'ratio_tree_sl.eps
    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'ratio_tree_sl.eps $HEAD'_'$RED'_'$OP'_'ratio_tree_sl.png

echo '   Ratio := real best (without vendor) / chosen best'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_best; grep 'best' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/PEs best/B__ divisor/' | cat - $HEAD'_'$RED'_'$OP'_'ByPeTy_best4 |  sed -e '1s/... by/000 by/' -e 's/PEs/C__/' | sort +0n -1 +2n -3 +3 -4 |  gawk -f ratio.awk | sed -e '1s/000/.../' -e 's/B__ divisor/PEs d:cb/' -e 's/.__/PEs/' -e '1s/best_of_4/methd ratio cat/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_best
    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_best; echo 'set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_ratio_best.eps" ; set title "'$RED_NAME'('$OP',dbl)  -  ratio := real best bandwidth of 4 new algo.s  /  chosen best" ; set yrange ['$PElow':'$PEhighT'] ;  load "comp5_ratio.gnuplot" ; q' | gnuplot
    \rm -f $HEAD'_'$RED'_'$OP'_'ratio_best_sl.eps ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'ratio_best.eps > $HEAD'_'$RED'_'$OP'_'ratio_best_sl.eps
    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'ratio_best_sl.eps $HEAD'_'$RED'_'$OP'_'ratio_best_sl.png

echo '   Ratio := real best of 5 / chosen best'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio5_best; grep 'best' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/PEs best/B__ divisor/' | cat - $HEAD'_'$RED'_'$OP'_'ByPeTy_best5 |  sed -e '1s/... by/000 by/' -e 's/PEs/C__/' | sort +0n -1 +2n -3 +3 -4 |  gawk -f ratio.awk | sed -e '1s/000/.../' -e 's/B__ divisor/PEs d:cb/' -e 's/.__/PEs/' -e '1s/best_of_5/methd ratio cat/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio5_best
    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_ratio5_best; echo 'set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_ratio5_best.eps" ; set title "'$RED_NAME'('$OP',dbl)  -  ratio := real best bandwidth of 4 new algo.s and vendor  /  chosen best" ; set yrange ['$PElow':'$PEhighT'] ;  load "comp5_ratio.gnuplot" ; q' | gnuplot
    \rm -f $HEAD'_'$RED'_'$OP'_'ratio5_best_sl.eps ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'ratio5_best.eps > $HEAD'_'$RED'_'$OP'_'ratio5_best_sl.eps
    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'ratio5_best_sl.eps $HEAD'_'$RED'_'$OP'_'ratio5_best_sl.png

echo '   Ratio := real best of 6 / chosen best'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio6_best; grep 'best' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/PEs best/B__ divisor/' | cat - $HEAD'_'$RED'_'$OP'_'ByPeTy_best6 |  sed -e '1s/... by/000 by/' -e 's/PEs/C__/' | sort +0n -1 +2n -3 +3 -4 |  gawk -f ratio.awk | sed -e '1s/000/.../' -e 's/B__ divisor/PEs d:cb/' -e 's/.__/PEs/' -e '1s/best_of_6/methd ratio cat/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio6_best
    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_ratio6_best; echo 'set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_ratio6_best.eps" ; set title "'$RED_NAME'('$OP',dbl)  -  ratio := real best measured bandwidth  /  chosen best" ; set yrange ['$PElow':'$PEhighT'] ;  load "comp5_ratio.gnuplot" ; q' | gnuplot
    \rm -f $HEAD'_'$RED'_'$OP'_'ratio6_best_sl.eps ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'ratio6_best.eps > $HEAD'_'$RED'_'$OP'_'ratio6_best_sl.eps
    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'ratio6_best_sl.eps $HEAD'_'$RED'_'$OP'_'ratio6_best_sl.png

#-------------------
#   over CPUs
#-------------------

echo '   Plots over CPUs' 
echo '     Generating comp5_overCPUs_'$OP'.gnuplot'
 
cat comp5_overCPUs_head.gnuplot > comp5_overCPUs_$OP.gnuplot
for BYs in `grep 'By  *'$PEmin' PEs best' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/ *//' -e 's/ By.*//'` 
do
  sed -e "s/bbbb/$BYs/" -e "s/rrrroooo/$RED_NAME($OP,dbl)/" comp5_red_overCPUs_entry.gnuplot >> comp5_overCPUs_$OP.gnuplot
done 
 
echo '     Generating plots'

#   For Paper:
      sed -e 's/rrrroooo/'$RED_NAME'('$OP',dbl)/' 'comp5_overCPUs_'$OP'.gnuplot' > xxxxcomp5_overCPUs.gnuplot 
      export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy; echo 'set xrange ['$PElow':'$PEhigh']; set logscale x 2; set term postscript eps enhanced color solid 22 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_overCPUs.eps" ; load "xxxxcomp5_overCPUs.gnuplot" ; q' | gnuplot
      \rm -f xxxxcomp5_overCPUs.gnuplot 
 

#   For slides:
      \rm -f $HEAD'_'$RED'_'$OP'_'overCPUs_sl.eps; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'overCPUs.eps > $HEAD'_'$RED'_'$OP'_'overCPUs_sl.eps
      # convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'overCPUs_sl.eps $HEAD'_'$RED'_'$OP'_'overCPUs_sl.png

#---------------------------
#   over CPUs - single plots
#---------------------------

echo '   Single Plots over CPUs' 
 
    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy 

    if [[ ( $BYmin == "8" ) || ( $BYmin == "16" ) ]] 
    then 
      export LNG=16;      export TLNG='16 bytes' ; ./analysis_1pic.sh
      export LNG=1024;    export TLNG='1 kb'     ; ./analysis_1pic.sh 
      export LNG=2048;    export TLNG='2 kb'     ; ./analysis_1pic.sh 
      export LNG=32768;   export TLNG='32 kb'    ; ./analysis_1pic.sh 
      export LNG=524288;  export TLNG='512 kb'   ; ./analysis_1pic.sh 
      export LNG=4194304; export TLNG='4 Mb'     ; ./analysis_1pic.sh 
    else 
      if [[ $BYmin == "12"  ]]
      then 
        export LNG=12;      export TLNG='12 bytes' ; ./analysis_1pic.sh
        export LNG=1020;    export TLNG='1 kb'     ; ./analysis_1pic.sh 
        export LNG=2040;    export TLNG='2 kb'     ; ./analysis_1pic.sh 
        export LNG=32760;   export TLNG='32 kb'    ; ./analysis_1pic.sh 
        export LNG=524280;  export TLNG='512 kb'   ; ./analysis_1pic.sh 
        export LNG=4194300; export TLNG='4 Mb'     ; ./analysis_1pic.sh 
      fi 
    fi 

# code inside of analysis_1pic.sh : 
#     echo '    LNG = '$TLNG' = '$LNG' bytes' 
#     echo 'set title "buffersize = '$TLNG'"; set xrange ['$PElow':'$PEhigh']; set logscale x 2; set term postscript eps enhanced color solid 30 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_overCPUs_'$LNG'bytes.eps" ; load "comp5_red_overCPUs_1pic.gnuplot" ; q' | gnuplot
#     \rm -f $HEAD'_'$RED'_'$OP'_'$LNG'bytes_sl.eps' ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_overCPUs_'$LNG'bytes.eps' > $HEAD'_'$RED'_'$OP'_overCPUs_'$LNG'bytes_sl.eps'
#     convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_overCPUs_'$LNG'bytes_sl.eps' $HEAD'_'$RED'_'$OP'_overCPUs_'$LNG'bytes_sl.png'

#-------------------
#   over Bytes
#-------------------

echo '   Plots over Bytes' 
echo '     Generating comp5_overBytes.gnuplot'
 
cat comp5_overBytes_head.gnuplot > comp5_overBytes.gnuplot
for PEs in `grep ' 1. By .... PEs best' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/.* By *//' -e 's/ PEs.*//'`
do
  sed -e "s/xxxx/$PEs/" -e "s/rrrroooo/$RED_NAME($OP,dbl)/" comp5_red_overBytes_entry.gnuplot >> comp5_overBytes.gnuplot
done 
 
echo '     Generating plots'

    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy;  echo 'set xrange [4:16777216];  set logscale x 2; set term postscript eps enhanced color solid 30 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_overBytes.eps" ; load "comp5_overBytes.gnuplot" ; q' | gnuplot
    \rm -f $HEAD'_'$RED'_'$OP'_'overBytes_sl.eps; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'overBytes.eps > $HEAD'_'$RED'_'$OP'_'overBytes_sl.eps
    # convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'overBytes_sl.eps $HEAD'_'$RED'_'$OP'_'overBytes_sl.png

# =======================================================================
else 
# RED = all

#-------------------
#   best algorithm out of 7:  (see %7 in the following command)
#-------------------

echo '   Best algorithm out of 7'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_best7; sed -e '1s/method/a_method/' $HEAD'_'$RED'_'$OP'_'ByPeTy |  sort +0n -1 +2n -3 +5nr -6 | gawk '{ if ((FNR-1)%7==1 || FNR==1) print $0 }' | sort +4 -5 +0n -1 +2n -3 | sed -e '1s/a_method/best_of_7/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_best7

#-------------------
#   best algorithm out of 6:  (see %6 in the following command)
#-------------------

echo '   Best algorithm out of 6'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_best6; sed -e '1s/method/a_method/' $HEAD'_'$RED'_'$OP'_'ByPeTy | grep -v 'PEs best' |  sort +0n -1 +2n -3 +5nr -6 | gawk '{ if ((FNR-1)%6==1 || FNR==1) print $0 }' | sort +4 -5 +0n -1 +2n -3 | sed -e '1s/a_method/best_of_6/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_best6

    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_best6 ; echo 'L2to32=11.25; L23to256=10.25; Lpair=13.5; set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_best.eps" ; set title "Fastest Protocol for\n'$RED_NAME'('$OP',dbl)" ; set yrange [1:'$PEmax10'] ; load "comp5_all_best.gnuplot" ; q' | gnuplot

    \rm -f $HEAD'_'$RED'_'$OP'_'best_sl.eps; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'best.eps > $HEAD'_'$RED'_'$OP'_'best_sl.eps
    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'best_sl.eps $HEAD'_'$RED'_'$OP'_'best_sl.png

#-------------------
#   Ratio best bandwidth of my 5 algorithms / vendor bandwidth
#-------------------

echo '   Best of new 5 algorithms'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_best5; sed -e '/best/d' -e '/dflt/d' -e '1s/method/a_method/' $HEAD'_'$RED'_'$OP'_'ByPeTy |  sort +0n -1 +2n -3 +5nr -6 | gawk '{ if ((FNR-1)%5==1 || FNR==1) print $0 }' | sort +4 -5 +0n -1 +2n -3 | sed -e '1s/a_method/best_of_5/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_best5

echo '   Ratio := real best / default'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_dflt; grep 'dflt' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/PEs ..../B__ divisor/' | cat - $HEAD'_'$RED'_'$OP'_'ByPeTy_best5 |  sed -e '1s/... by/000 by/' -e 's/PEs/C__/' | sort +0n -1 +2n -3 +3 -4 | gawk -f ratio.awk |  sed -e '1s/000/.../' -e 's/B__ divisor/PEs d:df/' -e 's/.__/PEs/' -e '1s/best_of_5/methd ratio cat/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_dflt

    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_dflt; echo 'set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_ratio_dflt.eps" ; set title "'$RED_NAME'('$OP',dbl)  -  ratio := best bandwidth of 5 new algo.s / vendor'"'"'s bandwidth" ; set yrange ['$PElow':'$PEhighT'] ; load "comp5_ratio.gnuplot" ; q' | gnuplot

    \rm -f $HEAD'_'$RED'_'$OP'_'ratio_dflt_sl.eps ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'ratio_dflt.eps > $HEAD'_'$RED'_'$OP'_'ratio_dflt_sl.eps

    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'ratio_dflt_sl.eps $HEAD'_'$RED'_'$OP'_'ratio_dflt_sl.png


echo '   Ratio := chosen best / default'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_cb_dflt; egrep 'dflt|best|... by' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/PEs dflt/B__ divisor/' -e '1s/... by/000 by/' -e 's/PEs/C__/' | sort +0n -1 +2n -3 +3 -4 | gawk -f ratio.awk |  sed -e '1s/000/.../' -e 's/B__ divisor/PEs d:df/' -e 's/.__/PEs/' -e '1s/method/methd ratio cat/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_cb_dflt

    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_cb_dflt; echo 'set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_ratio_cb_dflt.eps" ; set title "'$RED_NAME'('$OP',dbl)  -  ratio := bandwidth of chosen best / vendor'"'"'s bandwidth" ; set yrange ['$PElow':'$PEhighT'] ; load "comp5_ratio.gnuplot" ; q' | gnuplot

    \rm -f $HEAD'_'$RED'_'$OP'_'ratio_cb_dflt_sl.eps ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'ratio_cb_dflt.eps > $HEAD'_'$RED'_'$OP'_'ratio_cb_dflt_sl.eps

    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'ratio_cb_dflt_sl.eps $HEAD'_'$RED'_'$OP'_'ratio_cb_dflt_sl.png


echo '   Ratio := real best / tree'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_tree; grep 'tree' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/PEs tree/B__ divisor/' | cat - $HEAD'_'$RED'_'$OP'_'ByPeTy_best5 |  sed -e '1s/... by/000 by/' -e 's/PEs/C__/' | sort +0n -1 +2n -3 +3 -4 |  gawk -f ratio.awk | sed -e '1s/000/.../' -e 's/B__ divisor/PEs d:tr/' -e 's/.__/PEs/' -e '1s/best_of_5/methd ratio cat/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_tree

    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_tree; echo 'set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_ratio_tree.eps" ; set title "'$RED_NAME'('$OP',dbl)  -  ratio := best bandwidth of 5 new algo.s / bin-tree'"'"'s bandwidth" ; set yrange ['$PElow':'$PEhighT'] ;  load "comp5_ratio.gnuplot" ; q' | gnuplot

    \rm -f $HEAD'_'$RED'_'$OP'_'ratio_tree_sl.eps ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'ratio_tree.eps > $HEAD'_'$RED'_'$OP'_'ratio_tree_sl.eps

    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'ratio_tree_sl.eps $HEAD'_'$RED'_'$OP'_'ratio_tree_sl.png

echo '   Ratio := real best (without vendor) / chosen best'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_best; grep 'best' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/PEs best/B__ divisor/' | cat - $HEAD'_'$RED'_'$OP'_'ByPeTy_best5 |  sed -e '1s/... by/000 by/' -e 's/PEs/C__/' | sort +0n -1 +2n -3 +3 -4 |  gawk -f ratio.awk | sed -e '1s/000/.../' -e 's/B__ divisor/PEs d:cb/' -e 's/.__/PEs/' -e '1s/best_of_5/methd ratio cat/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_best

    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_ratio_best; echo 'set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_ratio_best.eps" ; set title "'$RED_NAME'('$OP',dbl)  -  ratio := real best bandwidth of 5 new algo.s  /  chosen best" ; set yrange ['$PElow':'$PEhighT'] ;  load "comp5_ratio.gnuplot" ; q' | gnuplot

    \rm -f $HEAD'_'$RED'_'$OP'_'ratio_best_sl.eps ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'ratio_best.eps > $HEAD'_'$RED'_'$OP'_'ratio_best_sl.eps

    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'ratio_best_sl.eps $HEAD'_'$RED'_'$OP'_'ratio_best_sl.png

echo '   Ratio := real best of 6 / chosen best'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio6_best; grep 'best' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/PEs best/B__ divisor/' | cat - $HEAD'_'$RED'_'$OP'_'ByPeTy_best6 |  sed -e '1s/... by/000 by/' -e 's/PEs/C__/' | sort +0n -1 +2n -3 +3 -4 |  gawk -f ratio.awk | sed -e '1s/000/.../' -e 's/B__ divisor/PEs d:cb/' -e 's/.__/PEs/' -e '1s/best_of_6/methd ratio cat/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio6_best

    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_ratio6_best; echo 'set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_ratio6_best.eps" ; set title "'$RED_NAME'('$OP',dbl)  -  ratio := real best bandwidth of 5 new algo.s and vendor  /  chosen best" ; set yrange ['$PElow':'$PEhighT'] ;  load "comp5_ratio.gnuplot" ; q' | gnuplot

    \rm -f $HEAD'_'$RED'_'$OP'_'ratio6_best_sl.eps ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'ratio6_best.eps > $HEAD'_'$RED'_'$OP'_'ratio6_best_sl.eps

    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'ratio6_best_sl.eps $HEAD'_'$RED'_'$OP'_'ratio6_best_sl.png

echo '   Ratio := real best of 7 / chosen best'

    \rm -f $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio7_best; grep 'best' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/PEs best/B__ divisor/' | cat - $HEAD'_'$RED'_'$OP'_'ByPeTy_best7 |  sed -e '1s/... by/000 by/' -e 's/PEs/C__/' | sort +0n -1 +2n -3 +3 -4 |  gawk -f ratio.awk | sed -e '1s/000/.../' -e 's/B__ divisor/PEs d:cb/' -e 's/.__/PEs/' -e '1s/best_of_7/methd ratio cat/' > $HEAD'_'$RED'_'$OP'_'ByPeTy_ratio7_best

    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy_ratio7_best; echo 'set term postscript eps enhanced color solid 24 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_ratio7_best.eps" ; set title "'$RED_NAME'('$OP',dbl)  -  ratio := real best measured bandwidth  /  chosen best" ; set yrange ['$PElow':'$PEhighT'] ;  load "comp5_ratio.gnuplot" ; q' | gnuplot

    \rm -f $HEAD'_'$RED'_'$OP'_'ratio7_best_sl.eps ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'ratio7_best.eps > $HEAD'_'$RED'_'$OP'_'ratio7_best_sl.eps

    convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'ratio7_best_sl.eps $HEAD'_'$RED'_'$OP'_'ratio7_best_sl.png

#-------------------
#   over CPUs
#-------------------

echo '   Plots over CPUs' 
echo '     Generating comp5_overCPUs_'$OP'.gnuplot'
 
cat comp5_overCPUs_head.gnuplot > comp5_overCPUs_$OP.gnuplot
for BYs in `grep 'By  *'$PEmin' PEs best' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/ *//' -e 's/ By.*//'` 
do
  sed -e "s/bbbb/$BYs/" -e "s/rrrroooo/$RED_NAME($OP,dbl)/" comp5_all_overCPUs_entry.gnuplot >> comp5_overCPUs_$OP.gnuplot
done 
 
echo '     Generating plots'

#   For Paper:
      sed -e 's/rrrroooo/'$RED_NAME'('$OP',dbl)/' 'comp5_overCPUs_'$OP'.gnuplot' > xxxxcomp5_overCPUs.gnuplot 
      export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy; echo 'set xrange ['$PElow':'$PEhigh']; set logscale x 2; set term postscript eps enhanced color solid 22 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_overCPUs.eps" ; load "xxxxcomp5_overCPUs.gnuplot" ; q' | gnuplot
      \rm -f xxxxcomp5_overCPUs.gnuplot 

#   For slides:
      \rm -f $HEAD'_'$RED'_'$OP'_'overCPUs_sl.eps; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'overCPUs.eps > $HEAD'_'$RED'_'$OP'_'overCPUs_sl.eps
      # convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'overCPUs_sl.eps $HEAD'_'$RED'_'$OP'_'overCPUs_sl.png

#---------------------------
#   over CPUs - single plots
#---------------------------

echo '   Single Plots over CPUs' 
 
    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy 

    if [[ ( $BYmin == "8" ) || ( $BYmin == "16" ) ]] 
    then 
      export LNG=16;      export TLNG='16 bytes' ; ./analysis_1pic.sh
      export LNG=1024;    export TLNG='1 kb'     ; ./analysis_1pic.sh 
      export LNG=2048;    export TLNG='2 kb'     ; ./analysis_1pic.sh 
      export LNG=32768;   export TLNG='32 kb'    ; ./analysis_1pic.sh 
      export LNG=524288;  export TLNG='512 kb'   ; ./analysis_1pic.sh 
      export LNG=4194304; export TLNG='4 Mb'     ; ./analysis_1pic.sh 
    else 
      if [[ $BYmin == "12"  ]]
      then 
        export LNG=12;      export TLNG='12 bytes' ; ./analysis_1pic.sh
        export LNG=1020;    export TLNG='1 kb'     ; ./analysis_1pic.sh 
        export LNG=2040;    export TLNG='2 kb'     ; ./analysis_1pic.sh 
        export LNG=32760;   export TLNG='32 kb'    ; ./analysis_1pic.sh 
        export LNG=524280;  export TLNG='512 kb'   ; ./analysis_1pic.sh 
        export LNG=4194300; export TLNG='4 Mb'     ; ./analysis_1pic.sh 
      fi 
    fi 

# code inside of analysis_1pic.sh : 
#     echo '    LNG = '$TLNG' = '$LNG' bytes' 
#     echo 'set title "buffersize = '$TLNG'"; set xrange ['$PElow':'$PEhigh']; set logscale x 2; set term postscript eps enhanced color solid 30 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_overCPUs_'$LNG'bytes.eps" ; load "comp5_all_overCPUs_1pic.gnuplot" ; q' | gnuplot
#     \rm -f $HEAD'_'$RED'_'$OP'_'$LNG'bytes_sl.eps' ; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_overCPUs_'$LNG'bytes.eps' > $HEAD'_'$RED'_'$OP'_overCPUs_'$LNG'bytes_sl.eps'
#     convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_overCPUs_'$LNG'bytes_sl.eps' $HEAD'_'$RED'_'$OP'_overCPUs_'$LNG'bytes_sl.png'

#-------------------
#   over Bytes
#-------------------

echo '   Plots over Bytes' 
echo '     Generating comp5_overBytes.gnuplot'
 
cat comp5_overBytes_head.gnuplot > comp5_overBytes.gnuplot
for PEs in `grep ' 1. By .... PEs best' $HEAD'_'$RED'_'$OP'_'ByPeTy | sed -e 's/.* By *//' -e 's/ PEs.*//'`
do
  sed -e "s/xxxx/$PEs/" -e "s/rrrroooo/$RED_NAME($OP,dbl)/" comp5_all_overBytes_entry.gnuplot >> comp5_overBytes.gnuplot
done 
 
echo '     Generating plots'

    export INP=$HEAD'_'$RED'_'$OP'_'ByPeTy;  echo 'set xrange [4:16777216];  set logscale x 2; set term postscript eps enhanced color solid 30 ; set size 1.5,1.5 ; set output "'$HEAD'_'$RED'_'$OP'_overBytes.eps" ; load "comp5_overBytes.gnuplot" ; q' | gnuplot

    \rm -f $HEAD'_'$RED'_'$OP'_'overBytes_sl.eps; sed -e 's/gnulinewidth 5.000/gnulinewidth 15.000/' $HEAD'_'$RED'_'$OP'_'overBytes.eps > $HEAD'_'$RED'_'$OP'_'overBytes_sl.eps
    # convert -geometry "130%x130%" $HEAD'_'$RED'_'$OP'_'overBytes_sl.eps $HEAD'_'$RED'_'$OP'_'overBytes_sl.png

# ==============================================================
fi
# RED = 'red' or 'all' 

#-------------------
#   End of Analysis loops
#-------------------

 done 
done
